/* LinkPool.java	- (c) Ilpo Jrvinen (ijjarvin@cs.helsinki.fi)
 *
 * Ohjelmoinnin Harjoitusty kevt 2000
 *
 * -tabsize 4
 */
 
/**
 * LinkPool
 * @author Ilpo Jrvinen <ijjarvin@cs.helsinki.fi>
 * @version 0.9 04.04.2000
 */
 
import java.util.*;					// Hashtable & Enumeration


public class LinkPool {

	private Hashtable links;				// link storage...


/***************************************************************************
 * Creates the link "pool".
 */
	public LinkPool()
	{
		links = new Hashtable( 32 );
	}


/***************************************************************************
 * Adds specified url to pool of <b>Link</b>s if not already in the pool. 
 *
 * @return true if url was in the pool or was inserted into it.
 */
	public boolean addLink( String url )
	{
		if (url == null || url.indexOf(':') < 1)		// check if reference is relative, (=we will not be able to solve it) 
			return false;								// or invalid ":dfsaas" protocol! Also protects againts some braindmg cases!

		if (links.containsKey( url ))					// check if key is already in the pool
			return true;
		
		String key = url;								// save the key, because the address could be modified...

		if (url.lastIndexOf('?') > url.lastIndexOf('/')) {		// clear "...?fdsafsad" tails! This is not FOOLPROOF!!! But better
			url = url.substring( 0, url.lastIndexOf('?') );		// than nothing... There might be some other GOTCHAs like this (in
		}														// HTML format) which I'm not aware of... :(
																
		links.put( key, new Link( url ) );				// create & insert to pool
		return true;
	}
	

	
	
/*****************************************************************************
 * Check whether the specified url is broken.
 *
 * @return true if link is broken.
 */
	public boolean isLinkBroken( String url ) 
	{
		if (url == null)								// shouldn't ever happend
			return false;
		
		if (!links.containsKey( url ))
			return false;								// braindmg caller

		Link l = (Link)links.get( url );
		return l.isBroken();
	}


/*****************************************************************************
 * Check whether the specified url is moved.
 *
 * @return true if link has moved.
 */
	public boolean isLinkMoved( String url ) 
	{
		if (url == null)								// shouldn't ever happend
			return false;
		
		if (!links.containsKey( url ))
			return false;								// braindmg caller

		Link l = (Link)links.get( url );
		return l.isMoved();
	}
	

/*****************************************************************************
 * Check whether the specified url is in known state..
 *
 * @return true if state of connection to url is known.
 */	
	public boolean isLinkFinished( String url )
	{
		if (url == null)								// shouldn't ever happend
			return true;
		
		if (!links.containsKey( url ))
			return true;								// braindmg caller
		
		Link l = (Link)links.get( url );
		return l.isFinished();
	}

/* ********************************************************************** */
/* toString - FOR DEBUGGING ONLY!!!!
 *
 */
	public String toString() 
	{
		String s = "";
		for (Enumeration e = links.elements(); e.hasMoreElements(); ) {
			s += e.nextElement() + "\n";
		}
		return s;	
	}

}
