/* LinkVerifier.java	- (c) Ilpo Jrvinen (ijjarvin@cs.helsinki.fi)
 *
 * Ohjelmoinnin harjoitusty kevt 2000
 *
 * -tabsize 4
 */
 
/**
 * LinkVerifier
 * @author Ilpo Jrvinen
 * @version 0.98
 */

import java.util.Vector;
import java.util.Enumeration;

public class LinkVerifier {


	private Vector files;
	
	private static final int INTERVAL = 1000;				// wait 1000ms with intervals when in "wait-until-finished" - stage


/********************************************************************************
 * MAIN, only creates an instance of this class.
 */
	public static void main( String args[] ) 
	{
		LinkVerifier l = new LinkVerifier( args );
		
		System.exit( 0 );
	}


/*********************************************************************************
 * This method constructs the linkpool and userinterface which both are then
 * given to every instance of HtmlFile in doFileList method.
 */
	public LinkVerifier( String args[] ) 
	{
		UserInterface ui = new UserInterface( args );		// ui = userInterface
		LinkPool pool = new LinkPool();
		
		files = new Vector();								// create local list of files
		
		/*
		 * Do all files userinterface gives us.
		 */
		doFileList( ui, pool );
		/*
		 * No more files to do, wait until every unfinished file terminates.
		 */
		waitFiles();
	}


/**********************************************************************************
 *
 */
	private void doFileList( UserInterface ui, LinkPool pool ) 
	{
		String filelist[];

		/*
		 * Loop until userinterface gives null list of files (=termination signal)
		 */
		while ((filelist = ui.getFiles()) != null) {

			for (int i = 0; i < filelist.length; i++) {				
				/*
				 * First make sure same file will not get modified twice, at the same time!
				 * If it has already finished, then it can be started again...
				 *
				 * Java hack because of String.equals(HtmlFile) is called when
				 * Vector's contains() method is used... Verified in debugger!
				 */
				boolean javaHack = true;
				for (Enumeration e = files.elements(); e.hasMoreElements(); ) {
					HtmlFile h = (HtmlFile)e.nextElement();
					if (h.equals(filelist[i])) {			// do HtmlFile.equals(String) equality check, WORKS!!!
						javaHack = false;					// String.equals(HtmlFile) doesn't!
						break;
					}
				}
				if (javaHack) {
					/*
					 * Start evaluation of the file.
					 */
					HtmlFile h = new HtmlFile( filelist[i], pool, ui );
					files.addElement( h );					// add to local list of files
				}
			}

			/*
			 * remove completed from local list of files.
			 */
			for (Enumeration e = files.elements(); e.hasMoreElements(); ) {
				HtmlFile f = (HtmlFile)e.nextElement();
				if (f.isFinished())
					files.removeElement( f );				// can safely be removed because Iterator has passed by.
			}
		}
	}


/************************************************************************************
 * This method waits until unfinished files to complete.
 */
	private void waitFiles() 
	{
		for (Enumeration e = files.elements(); e.hasMoreElements(); ) {
			HtmlFile f = (HtmlFile)e.nextElement();
			while (!f.isFinished()) {
				try {
					Thread.currentThread().sleep( INTERVAL );
				} catch (InterruptedException ex) {}
			}
			files.removeElement( f );						// remove from list, Iterator survives!
		}
	}
}
