/* Timer.java	- (c) Ilpo Jrvinen (ijjarvin@cs.helsinki.fi)
 *
 * Ohjelmoinnin harjoitusty kevt 2000
 *
 * -tabsize 4
 */

/**
 * Timer
 * @author Ilpo Jrvinen <ijjarvin@cs.helsinki.fi>
 * @version 0.99 04.04.2000
 */
 
/* 04.04.2000:		Major reconstruction, timer functionality has changed
 *					Count can now be increased.
 */
 
 
public class Timer extends Thread {

/* ********************************************************************** */
	private long endMillisecs;				// end of wait.
	private int interval;					// how often the run() returns (ms)
	private int single;

/***************************************************************************
 * Constructs a timer with specified total counter and interval. The timer
 * "goes off" (=starts counting) during the call! Delayed triggering is
 * not possible.
 *
 * @param	min			Time (in secs) until timer expires.
 * @param	secs		Time (in secs) which will be added to counter during each add() call.
 * @param	interval	Maximal amount (in secs) of sleep per call to run.
 */
	public Timer( int min, int secs, int interval ) 
	{
		if (secs < 0)
			secs = 0;
		single = secs;
		this.interval = interval * 1000;
		if (interval < 0)
			interval = 1;
		if (min < 0)
			min = 0;
		endMillisecs = min * 1000 + System.currentTimeMillis();		// calculate the "finish-time"
		add();
	}
	
/**************************************************************************
 * Adds more time to counter
 */
	public void add()
	{
		endMillisecs += single * 1000;
	}

/***************************************************************************
 * This method just sleeps if the timer has not expired yet. The maximal
 * amount of sleep per call can be set with interval parameter when Timer
 * is constructed.
 */
	public void run() 
	{
		long milli = endMillisecs - System.currentTimeMillis();

		if (milli <= 0)
			return;
		
		try {
			sleep( milli > interval ? interval : milli );				// sleep if still some
		} catch( InterruptedException e ) {}
	}

/***************************************************************************
 * Returns expiration state of the timer.
 *
 * @return true if timer has expired.
 */
	public boolean isExpired() 			
	{ 
		return (endMillisecs - System.currentTimeMillis()) <= 0; 		// compare current time with the calculated "finish-time"
	}


/***************************************************************************
 * Returns <b>String</b> representation of the timer state.
 * FOR DEBUGGING ONLY!!!
 */
	public String toString() 
	{
		long milli = endMillisecs - System.currentTimeMillis();
		if (milli < 0)
			milli = 0;
		return milli + "ms left";
	}
}