/* UserInterface.java 	- (c) Ilpo Jrvinen (ijjarvin@cs.helsinki.fi)
 *
 * Ohjelmoinnin harjoitusty kevt 2000
 *
 * -tabsize 4
 */
 
/**
 * UserInterface
 * @author Ilpo Jrvinen <ijjarvin@cs.helsinki.fi>
 * @version 1.0 04.04.2000
 */

import java.util.*;								// Vector

public class UserInterface {

	private boolean 	help = false;			// do help screen?
	private boolean		version = false;		// do version screen?
	private boolean 	report = false; 		// report mode selected?
	private boolean 	reportAll = false;		// report-all mode selected?
	private boolean 	write = false;			// write mode selected? (default)

	private String 		fileList[];				// list of files to be checked
	
/* ************************************************************************ */
	public final int 	DEFAULT_TIMEOUT = 5;

	private int 		timeout;				// how long the checking of a single file can take. (in secs)
	
/* ************************************************************************ */
	private static final String usageString =	
"\n"+
"Usage:\n"+
"  java LinkVerifier [switches] [files]\n" +
"\n"+
"switches:\n"+
"  -H,--help\t\tPrint this information\n"+
"  -V,--version\t\tPrint version information\n"+
"  -r,--report\t\tDon't modify HTML, report broken links with status\n"+
"  -R,--report-all\tDon't modify HTML, report all links with status\n"+
"\n"+
"files\t\t\tFiles to verify\n";

	private static final String versionString = "\n"+
"LinkVerifier v0.98  Copyright (c) 2000 by Ilpo Jrvinen\n"+
"\n"+
"LinkVerifier comes with ABSOLUTELY NO WARRANTY; for details\n"+
"check the README file.  This is  free software, and you are\n"+
"welcome to redistribute it under certain conditions;  check\n"+
"the README for details.\n"+
"\n"+
"This application uses HTML parsing technology from Arthur Do Consulting.\n";

/* ************************************************************************ */
/* NOT USED */
/* ************************************************************************ */
	public UserInterface() 
	{
		this( null );
	}


/****************************************************************************
 * Creates the "user interface".
 */
	public UserInterface( String args[] ) 
	{
		timeout = DEFAULT_TIMEOUT;
		parseArgs( args );	
		if (!report && !reportAll) {
			write = true;
		}
	}
	

/****************************************************************************
 * Prints the help screen.
 */
	private void usage() 
	{
		println( usageString );
	}

/****************************************************************************
 * Prints the version screen.
 */
	private void version() 
	{
		println( versionString );
	}


/****************************************************************************
 * This method parses commandline arguments, which can be then queryed
 * using methods getFiles, reportSelect, reportAllSelected and writeSelected.
 */
	private void parseArgs( String args[] ) 
	{
		Vector fileLister = new Vector();							// create the temporary file holder
		if (args != null) {
			for (int i = 0; i < args.length; i++) {
				if (args[i] == null)								// kill bombs
					continue;
				
				String temp = args[i].trim();
				/*
				 * first check for switches...
				 */
				if (temp.equals( "-r" ) || temp.equals( "--report" )) {		
					report = true;
					args[i] = null;
				} else if (temp.equals( "-H" ) || temp.equals( "--help" )) {
					help = true;
					args[i] = null;
				} else if (temp.equals( "-V" ) || temp.equals( "--version" )) {
					version = true;
					args[i] = null;
				} else if (temp.equals( "-R" ) || temp.equals( "--report-all" )) {
					report = true;
					reportAll = true;
					args[i] = null;
				} else {
					/*
					 * It wasn't a switch, so assume it's a filename.
					 * No empty "" filenames will be allowed!
					 */
					if (args[i].length() > 0)
						fileLister.addElement(args[i]);
				}
			}
		}

		if (version)
			version();
		if (fileLister.size() == 0 || help) {
			if (version && !help) {}
			else
				usage();									// do help if necessary (= no files or a switch)
			fileList = null;								// if help was requested don't do any files.
		} else {
			fileList = new String[ fileLister.size() ];		// copy filelist from temporary holder to an array
			fileLister.copyInto( fileList );				// toArray() didn't work??? 1.2+???
		}
	}


/****************************************************************************
 * This method returns list of filenames which should be checked.
 */
	public String[] getFiles() 
	{
		String[] t = fileList;
		fileList = null;									// clear filelist (=don't do same files again)
		return t;
	}


/*
 * These methods are used for resultmode determination.
 */
	public boolean reportSelected() 	{ return report; }	
	public boolean writeSelected()		{ return write; }
	public boolean reportAllSelected()	{ return reportAll;	}
	

/**
 * All output will go through this or println method.
 * Current version is just a simple "redirector".
 */
	public synchronized void print( String out ) 
	{
		System.out.print( out );
	}
	
/**
 * All output will go through this or print method.
 * Current version is just a simple "redirector".
 */
	public synchronized void println( String out ) 
	{
		System.out.println( out );
	}


/**
 * This method returns how long a single file
 * must wait until it assumes remaining links to be broken.
 */
	public int getTimeout()				{ return timeout; }


}